/*
 * Project: MoleCuilder
 * Description: creates and alters molecular systems
 * Copyright (C)  2010-2012 University of Bonn. All rights reserved.
 * Please see the LICENSE file or "Copyright notice" in builder.cpp for details.
 */

/*
 * QtDialog.cpp
 *
 *  Created on: Jan 18, 2010
 *      Author: crueger
 */

// include config.h
#ifdef HAVE_CONFIG_H
#include <config.h>
#endif

#include "UIElements/Qt4/QtDialog.hpp"
#include "UIElements/Qt4/Query/QtQuery.hpp"

#include <QtGui/QDialogButtonBox>
#include <Qt/qpushbutton.h>
#include <Qt/qboxlayout.h>

#include "CodePatterns/MemDebug.hpp"


using namespace std;

QtDialog::QtDialog() :
    QDialog(0)
{
  // creating and filling of the Dialog window
  mainLayout = new QVBoxLayout();
  inputLayout = new QVBoxLayout();
  buttonLayout = new QVBoxLayout();
  setLayout(mainLayout);
  mainLayout->addLayout(inputLayout);
  mainLayout->addLayout(buttonLayout);
  buttons = new QDialogButtonBox(QDialogButtonBox::Ok| QDialogButtonBox::Cancel);
  buttonLayout->addWidget(buttons);

  // Disable the ok button until something was entered
  buttons->button(QDialogButtonBox::Ok)->setEnabled(false);

  // connect the buttons to their appropriate slots
  connect(buttons, SIGNAL(accepted()), this, SLOT(accept()));
  connect(buttons, SIGNAL(rejected()), this, SLOT(reject()));
}

QtDialog::~QtDialog()
{
}

bool QtDialog::display(){
  // Button state might have changed by some update that
  // was done during query construction. To make sure
  // the state is correct, we just call update one more time.
  update();
  if(exec()) {
    setAll();
    return true;
  }
  else {
    return false;
  }
}

void QtDialog::update(){
  buttons->button(QDialogButtonBox::Ok)->setEnabled(checkAll());
}

/************************** Query Infrastructure ************************/

void QtDialog::queryEmpty(const char* title, std::string)
{
  registerQuery(new EmptyQtQuery(title,inputLayout,this));
}

void QtDialog::queryBoolean(Parameter<bool> &param, const char* title,string)
{
  registerQuery(new BooleanQtQuery(param,title,inputLayout,this));
}

void QtDialog::queryAtom(Parameter<const atom *> &param, const char* title, std::string)
{
  registerQuery(new AtomQtQuery(param,title,inputLayout,this));
}

void QtDialog::queryAtoms(Parameter<std::vector<const atom *> > &param, const char* title, std::string)
{
  registerQuery(new AtomsQtQuery(param,title,inputLayout,this));
}

void QtDialog::queryRealSpaceMatrix(Parameter<RealSpaceMatrix> &param, const char* title, std::string)
{
  registerQuery(new RealSpaceMatrixQtQuery(param,title,inputLayout,this));
}

void QtDialog::queryInt(Parameter<int> &param, const char *title,string)
{
  registerQuery(new IntQtQuery(param,title,inputLayout,this));
}

void QtDialog::queryInts(Parameter<std::vector<int> > &param, const char *title,string)
{
  registerQuery(new IntsQtQuery(param,title,inputLayout,this));
}

void QtDialog::queryUnsignedInt(Parameter<unsigned int> &param, const char *title,string)
{
  registerQuery(new UnsignedIntQtQuery(param,title,inputLayout,this));
}

void QtDialog::queryUnsignedInts(Parameter<std::vector<unsigned int> > &param, const char *title,string)
{
  registerQuery(new UnsignedIntsQtQuery(param,title,inputLayout,this));
}

void QtDialog::queryDouble(Parameter<double> &param, const char* title,string)
{
  registerQuery(new DoubleQtQuery(param,title,inputLayout,this));
}

void QtDialog::queryDoubles(Parameter<std::vector<double> > &param, const char* title,string)
{
  registerQuery(new DoublesQtQuery(param,title,inputLayout,this));
}

void QtDialog::queryString(Parameter<std::string> &param, const char* title,string)
{
  registerQuery(new StringQtQuery(param,title,inputLayout,this));
}

void QtDialog::queryStrings(Parameter<std::vector<std::string> > &param, const char* title,string)
{
  registerQuery(new StringsQtQuery(param, title,inputLayout,this));
}

void QtDialog::queryMolecule(Parameter<const molecule *> &param, const char *title,string)
{
  registerQuery(new MoleculeQtQuery(param,title,inputLayout,this));
}

void QtDialog::queryMolecules(Parameter<std::vector<const molecule *> > &param, const char *title,string)
{
  registerQuery(new MoleculesQtQuery(param, title,inputLayout,this));
}

void QtDialog::queryVector(Parameter<Vector> &param, const char* title, bool check,string)
{
  registerQuery(new VectorQtQuery(param,title,check,inputLayout,this));
}

void QtDialog::queryVectors(Parameter<std::vector<Vector> > &param, const char* title, bool check,string)
{
  registerQuery(new VectorsQtQuery(param, title,check,inputLayout,this));
}

void QtDialog::queryElement(Parameter<const element *> &param, const char* title, std::string)
{
  registerQuery(new ElementQtQuery(param,title,inputLayout,this));
}

void QtDialog::queryElements(Parameter<std::vector<const element *> > &param, const char* title, std::string)
{
  registerQuery(new ElementsQtQuery(param,title,inputLayout,this));
}

void QtDialog::queryFile(Parameter<boost::filesystem::path> &param, const char* title, std::string)
{
  registerQuery(new FileQtQuery(param,title,inputLayout,this));
}

void QtDialog::queryRandomNumberDistribution_Parameters(Parameter<RandomNumberDistribution_Parameters> &param, const char* title, std::string)
{
  registerQuery(new RandomNumberDistribution_ParametersQtQuery(param,title,inputLayout,this));
}

/************************** Query Infrastructure ************************/
/*       ---> shifted to folder Query                                   */
/************************************************************************/

/*************************** Pipe Infrastructure ************************/
/*       ---> shifted to folder Pipe                                    */
/************************************************************************/
